#!/usr/bin/python3

from struct import pack
from tinytag import TinyTag

from .. import SmallTag

if True:
    from collections.abc import Callable, Iterator  # noqa
    from typing import Any, BinaryIO, Dict, List


_tinytag_id3 = TinyTag._get_parser_for_filename(".mp3")  # noqa


class _ID3(SmallTag, _tinytag_id3):
    @staticmethod
    def _synchsafe(unsynchsafe_int: int) -> tuple[int, ...]:
        ints = (
            (unsynchsafe_int >> 21) & 0x7F,
            (unsynchsafe_int >> 14) & 0x7F,
            (unsynchsafe_int >> 7) & 0x7F,
            unsynchsafe_int & 0x7F
        )

        return ints

    def write(self, file_obj: BinaryIO=None):
        should_close_file = self._set_filehandler_for_writing(file_obj)

        # change keys to values in the mapping dict
        self._ID3_WRITE_MAPPING = {}  # noqa
        for frame_id, name in self._ID3_MAPPING.items():
            if len(frame_id) == 4:
                self._ID3_WRITE_MAPPING[name] = frame_id

        new_frames = self._compose_id3v2_frames()

        size = len(new_frames)

        new_header = self._compose_id3v2_header(size)

        new_tag = new_header + new_frames
        print(new_tag)

        size, extended, major = self._parse_id3v2_header(self._filehandler)

        file_content = self._filehandler.read()
        audio = file_content[size:]

        self._filehandler.seek(0)
        self._filehandler.write(new_tag + audio)
        self._filehandler.truncate()

        if should_close_file:
            self._filehandler.close()

    def _compose_id3v2_header(self, size: int):
        header = b"ID3\x04\x00\x00"

        synchsafe_size = self._synchsafe(size)
        header += pack("4B", *synchsafe_size)

        return header

    def _compose_id3v2_frames(self):
        tag_dict = self.as_dict()

        frames = b""

        for field_name, field_value in tag_dict.items():
            if field_name not in self._ID3_WRITE_MAPPING:
                continue

            frames += self._compose_id3v2_frame(field_name, field_value)

        return frames

    def _compose_id3v2_frame(self, field_name, field_value):
        frame_id = bytes(self._ID3_WRITE_MAPPING[field_name], "ISO-8859-1")

        field_value = field_value[0]

        if isinstance(field_value, str):
            frame_value = b"\x00" + bytes(field_value, "ISO-8859-1")

        else:
            print(f"Writing of {field_name} is not implemented.")
            return b""

        frame_size = pack("4B", *self._synchsafe(len(frame_value)))

        frame_data = frame_id + frame_size + b"\x00\x00" + frame_value

        return frame_data
